@@ -58,7 +58,7 @@
        Keith Owens <kaos@ocs.com.au> September 2001.
    */
 
-#ident "$Id: insmod.c 1.4.1.5 Sun, 11 Nov 2001 15:54:54 +1100 kaos $"
+#ident "$Id: insmod.c 1.4.1.8 Tue, 20 Nov 2001 14:09:11 +1100 kaos $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -94,6 +94,7 @@ static int flag_ksymoops = 1;
 
 static int n_ext_modules_used;
 static int m_has_modinfo;
+static int gplonly_seen;
 
 extern int insmod_main(int argc, char **argv);
 extern int insmod_main_32(int argc, char **argv);
@@ -255,6 +256,7 @@ static int add_symbols_from(struct obj_file *f, int idx,
 		 * their references.
 		 */
 		if (strncmp((char *)s->name, "GPLONLY_", 8) == 0) {
+			gplonly_seen = 1;
 			if (gpl)
 				((char *)s->name) += 8;
 			else
@@ -1413,7 +1415,7 @@ static void set_tainted(struct obj_file *f, int fd, int kernel_has_tainted,
 /* Check if loading this module will taint the kernel. */
 static void check_tainted_module(struct obj_file *f, int noload)
 {
-	static const char tainted_file[] = "/proc/sys/kernel/tainted";
+	static const char tainted_file[] = TAINT_FILENAME;
 	int fd, kernel_has_tainted;
 	const char *ptr;
 
@@ -1434,22 +1436,22 @@ static void check_tainted_module(struct obj_file *f, int noload)
 	case 0:
 		break;
 	case 1:
-		set_tainted(f, fd, kernel_has_tainted, noload, 1, "no license", "");
+		set_tainted(f, fd, kernel_has_tainted, noload, TAINT_PROPRIETORY_MODULE, "no license", "");
 		break;
 	case 2:
 		/* The module has a non-GPL license so we pretend that the
 		 * kernel always has a taint flag to get a warning even on
 		 * kernels without the proc flag.
 		 */
-		set_tainted(f, fd, 1, noload, 1, "non-GPL license - ", ptr);
+		set_tainted(f, fd, 1, noload, TAINT_PROPRIETORY_MODULE, "non-GPL license - ", ptr);
 		break;
 	default:
-		set_tainted(f, fd, 1, noload, 1, "Unexpected return from obj_gpl_license", "");
+		set_tainted(f, fd, 1, noload, TAINT_PROPRIETORY_MODULE, "Unexpected return from obj_gpl_license", "");
 		break;
 	}
 
 	if (flag_force_load)
-		set_tainted(f, fd, 1, noload, 2, "forced load", "");
+		set_tainted(f, fd, 1, noload, TAINT_FORCED_MODULE, "forced load", "");
 	if (fd >= 0)
 		close(fd);
 }
@@ -1703,7 +1705,7 @@ int INSMOD_MAIN(int argc, char **argv)
 	 */
 	set_ncv_prefix(NULL);
 
-	for (i = 0; i < n_module_stat; ++i) {
+	for (i = 0; !noload && i < n_module_stat; ++i) {
 		if (strcmp(module_stat[i].name, m_name) == 0) {
 			error("a module named %s already exists", m_name);
 			goto out;
@@ -1771,8 +1773,20 @@ int INSMOD_MAIN(int argc, char **argv)
 
 	arch_create_got(f);     /* DEPMOD */
 	if (!obj_check_undefineds(f, quiet)) {	/* DEPMOD, obj_clear_undefineds */
-		if (!gpl && !quiet)
-			error("Note: modules without a GPL compatible license cannot use GPLONLY_ symbols");
+		if (!gpl && !quiet) {
+			if (gplonly_seen)
+				error("\n"
+				      "Hint: You are trying to load a module without a GPL compatible license\n"
+				      "      and it has unresolved symbols.  The module may be trying to access\n"
+				      "      GPLONLY symbols but the problem is more likely to be a coding or\n"
+				      "      user error.  Contact the module supplier for assistance, only they\n"
+				      "      can help you.\n");
+			else
+				error("\n"
+				      "Hint: You are trying to load a module without a GPL compatible license\n"
+				      "      and it has unresolved symbols.  Contact the module supplier for\n"
+				      "      assistance, only they can help you.\n");
+		}
 		goto out;
 	}
 	obj_allocate_commons(f);	/* DEPMOD */
