@@ -977,7 +977,7 @@ static int insmod(DESC *desc, LINK **new_in_kernel, char *options[])
 
 	/* Something new for us to do? Or is the module already installed... */
 	if (desc == NULL || strcmp(desc->objkey, "null") == 0 ||
-	    lookup_key(in_kernel, desc->kname) != NULL)
+	    runit && lookup_key(in_kernel, desc->kname) != NULL)
 		return 0;
 	/* else */
 
