@@ -18,7 +18,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: xmalloc.c 1.3 Mon, 26 Feb 2001 15:16:48 +1100 kaos $"
+#ident "$Id: xmalloc.c 1.4 Sun, 06 May 2001 14:36:55 +1000 kaos $"
 
 #include <stdlib.h>
 #include "util.h"
@@ -29,7 +29,7 @@
 void *
 xmalloc(size_t size)
 {
-  void *ptr = malloc(size);
+  void *ptr = malloc(size ? size : 1);
   if (!ptr)
     {
       error("Out of memory");
